package com.lyms.talkonlineweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.lyms.talkonlineweb.annotation.TokenRequired;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.result.BaseResponse;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.Constant;
import com.lyms.talkonlineweb.util.HXService;
import com.lyms.talkonlineweb.util.StringUtil;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping("tk")
@Log4j2
public class TkRecordController {
    @Value("${hx.hxuser}")
    private String hxuser;
    @Autowired
    private LymsTkrecordService lymsTkrecordService;
    @Autowired
    private TkrecordInfoService tkrecordInfoService;

    @Autowired
    private LymsTcardService lymsTcardService;
    @Autowired
    private HXService hxService;
    @Autowired
    private LymsChatgroupService lymsChatgroupService;

    @Autowired
    private LymsDoctorService lymsDoctorService;
    @Autowired
    private LymsPatientService lymsPatientService;
    @Autowired
    private LymsReturnVisitRecordService lymsReturnVisitRecordService;

    /**
     * 保存问诊记录
     * @param tkrecord
     * @return
     */
    @PostMapping("saveTkRecord")
    @TokenRequired
    public BaseResponse saveTkRecord(@RequestBody @Validated LymsTkrecord tkrecord){
        BaseResponse baseResponse=new BaseResponse();
        try {
            if(null==tkrecord.getType()) {
                LymsTcard tcard = new LymsTcard();
                tcard.setPid(tkrecord.getPid());
                tcard.setPcid(tkrecord.getPcid());
//            获取问诊卡按照先从医院购买的问诊卡消费
                List<LymsTcard> rLst = lymsTcardService.list(Wrappers.query(tcard).notInSql("id", "SELECT r.cid FROM lyms_tkrecord r").orderByDesc("fid"));

//            小程序购买的问诊卡
                tcard.setPcid(null);
                tcard.setFid(1);
                List<LymsTcard> xLst = lymsTcardService.list(Wrappers.query(tcard).notInSql("id", "SELECT r.cid FROM lyms_tkrecord r").orderByDesc("fid"));

                if (rLst.size() > 0 || xLst.size() > 0) {
                    if (rLst.size() > 0) {
                        tkrecord.setCid(rLst.get(0).getId());
                        tkrecord.setFid((byte) 2);
                    } else {
                        tkrecord.setCid(xLst.get(0).getId());
                        tkrecord.setFid((byte) 1);
                    }

                } else {
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("已经没有了问诊卡");
                    return baseResponse;
                }
            }
            LymsChatgroup group =new LymsChatgroup();
            LymsDoctor doctor=lymsDoctorService.getById(tkrecord.getDid());
            LymsPatient patient=lymsPatientService.getById(tkrecord.getPid());

            List<LymsChatgroup> gLst = lymsChatgroupService.list(new QueryWrapper<LymsChatgroup>()
                .lambda().eq(LymsChatgroup::getPcid, tkrecord.getPcid()));
            //之前有这个病例的问诊就不用新创建组
            if (0==gLst.size()) {
                group.setOwnerk(hxuser);
                List<String> adminDlogins=new ArrayList<>();
                adminDlogins.add(patient.getIdno());
                List<String> groupnames=new ArrayList<>();
                groupnames.add(patient.getPname());
                //患者注册环信
                if (StringUtil.isEmpty(patient.getHxid())) {
                    JSONObject json = hxService.addUser(patient.getIdno(), Constant.COMMON_PASSWD, patient.getPname());
                    JSONArray rArr = json.getJSONArray("entities");
                    if (rArr.size() > 0) {
                        patient.setHxid(rArr.getJSONObject(0).getString("uuid"));
                        lymsPatientService.updateById(patient);
                    }
                }
                //查询值班医生/值班护士
                List<LymsDoctor> doctorAminList= lymsDoctorService.list(new QueryWrapper<LymsDoctor>()
                        .lambda().eq(LymsDoctor::getDpid, doctor.getDpid())
                        .in(LymsDoctor::getAdminType, Arrays.asList(1,2))
                        .orderByAsc(LymsDoctor::getAdminType));
                //科室值班医生注册环信
                for (LymsDoctor lymsDoctor : doctorAminList) {
                    if(StringUtil.isEmpty(lymsDoctor.getHxid())) {
                        JSONObject json = hxService.addUser(lymsDoctor.getDlogin(), Constant.COMMON_PASSWD, lymsDoctor.getDname());
                        JSONArray rArr = json.getJSONArray("entities");
                        if (rArr.size() > 0) {
                            lymsDoctor.setHxid(rArr.getJSONObject(0).getString("uuid"));
                            lymsDoctorService.updateById(lymsDoctor);
                        }
                    }
                    adminDlogins.add(lymsDoctor.getDlogin());
                    groupnames.add(lymsDoctor.getDname());
                }
                if ( ! adminDlogins.contains(doctor.getDlogin())) {
                    //医生注册环信
                    if (StringUtil.isEmpty(doctor.getHxid())) {
                        JSONObject json = hxService.addUser(doctor.getDlogin(), Constant.COMMON_PASSWD, doctor.getDname());
                        JSONArray rArr = json.getJSONArray("entities");
                        if (rArr.size() > 0) {
                            doctor.setHxid(rArr.getJSONObject(0).getString("uuid"));
                            lymsDoctorService.updateById(doctor);
                        }
                    }
                    adminDlogins.add(doctor.getDlogin());
                    groupnames.add(doctor.getDname());
                }


                log.info("创建环信组:{} description:{}", group.getOwnerk(), StringUtils.join(adminDlogins.toArray(), ","));
                JSONObject rJson = hxService.addChatGroups(StringUtils.join(adminDlogins.toArray(), ","), group.getOwnerk(), adminDlogins.toArray(new String[adminDlogins.size()]));
                log.info("rJson:" + rJson);
                group.setHxgroupid(rJson.getJSONObject("data").getString("groupid"));
                group.setDescription(StringUtils.join(adminDlogins.toArray(), ","));
                //环信群组名称更改为姓名串-用于PC端问诊展示
                group.setGroupname(StringUtils.join(groupnames.toArray(), ","));
                group.setFromp(patient.getIdno());
                group.setTarget(doctor.getDlogin());
                group.setPcid(tkrecord.getPcid());
                group.setHdepartId(doctor.getDpid());
                if(null==tkrecord.getType()) {
                    lymsChatgroupService.saveOrUpdate(group);
                    tkrecord.setHxgroupid(group.getHxgroupid());
                    lymsTkrecordService.save(tkrecord);
                }else {
                    group.setType(1);
                    lymsChatgroupService.saveOrUpdate(group);
                    LymsReturnVisitRecord returnVisitRecord=new LymsReturnVisitRecord();
                    returnVisitRecord.setDlogin(tkrecord.getRetDlogin());//前端传的当前登录医生账号
                    returnVisitRecord.setDpid(doctor.getDpid());
                    returnVisitRecord.setIdno(patient.getIdno());
                    returnVisitRecord.setType(1);
                    returnVisitRecord.setPcid(tkrecord.getPcid());
                    returnVisitRecord.setHxgroupid(group.getHxgroupid());
                    lymsReturnVisitRecordService.save(returnVisitRecord);
                }
            }else {
                group=gLst.get(0);
                tkrecord.setHxgroupid(group.getHxgroupid());
                lymsTkrecordService.save(tkrecord);
            }

            baseResponse.setObject(group);

            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }

        return baseResponse;
    }

    /**
     * 删除问诊记录
     * @param tkid
     * @return
     */
    @PostMapping("deleteTkRecord")
    @TokenRequired
    public BaseResponse deleteTkRecord(Integer tkid){
        BaseResponse baseResponse=new BaseResponse();
        try {
            lymsTkrecordService.removeById(tkid);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }

        return baseResponse;
    }

    /**
     * 医生端问诊统计
     * @param tkrecordInfo(需给did赋值)
     * @return 本月问诊数量themonth、上月问诊数量lastmonth、总问诊数量alltotal
     */
    @GetMapping("queryTkRecordStatisticsByDid")
    @TokenRequired
    public BaseResponse queryTkRecordStatisticsByDid(TkrecordInfo tkrecordInfo){
        BaseResponse baseResponse=new BaseResponse();
        try {
            baseResponse.setObject(tkrecordInfoService.getOne(Wrappers.query(tkrecordInfo)));
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }

        return baseResponse;
    }
}
